==> vdr-xxv_0.20.orig.tar.gz: xxv-0.20/lib/XXV/MODULES/TELNET.pm <==
package XXV::MODULES::TELNET;

use strict;

use XXV::OUTPUT::Console;
use IO::Socket::INET;
use Module::Reload;
use Tools;
use Locale::gettext;

# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'TELNET',
        Description => gettext('This module is a multisession telnet server.'),
        Version => '0.01',
        Date => '18.08.2004',
        Author => 'xpix',
        Preferences => {
            Clients => {
                description => gettext('Max allowed Connection in the same time'),
                default     => 5,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
            Port => {
                description => gettext('Portnumber to connect the client'),
                default     => 8005,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
        },
        Commands => {
            help => {
                description => gettext("This will display all commands or the helptext from the 'module name'"),
                short       => 'h',
                callback    => sub{
                    return $obj->usage(@_);
                },
            },
            quit => {
                description => gettext("This will exit the telnet session"),
                short       => 'q',
                callback    => sub{
                    my ($w, $c, $l) = @_;
                    warn   "[Server] Closed telnet session.\n";
                    $c->message(gettext("Closed session."));
                    delete $obj->{USER};
                    $w->w->fd->close;
                    $w->w->cancel;
                },
            },
            bye => {
                description => gettext("This will exit the xxv system."),
                short       => 'x',
                callback    => sub{
                    my ($w, $c, $l) = @_;
        		    my $answer;
        		    my $questions = [
            		    'really' =>	{
                                    typ => 'confirm',
                                    msg => gettext("Are you sure to exit the XXV System?"),
                                    def => 'n'}
        		    ];
                    $answer = $c->question(gettext("This will exit the xxv system."),$questions,$answer);
                    if($answer->{really} eq 'y') {
                        delete $obj->{USER};
                        $w->w->fd->close;
                        $w->w->cancel;
                        warn   "[Server] Closed session and exit.\n";
                        main::quit;
                    }
                },
        		Level   => 'admin'
            },
            reload => {
                description => gettext("This will reload all Modules."),
                short       => 're',
                callback    => sub{
                    my ($w, $c, $l) = @_;
                    $Module::Reload::Debug = 2;
                    Module::Reload->check;
                    $c->message(gettext("Modules reloaded."));
                },
		        Level       => 'user'
            },
        },
    };
    return $args;
}

# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_} || $self->{MOD}->{Preferences}->{$_}->{default}
    } keys %{$self->{MOD}->{Preferences}};

    # read the DB Handle
    $self->{dbh} = delete $attr{'-dbh'};

    # The Initprocess
    $self->init or return error('Problem to initialze Module');

	return $self;
}


# ------------------
sub init {
# ------------------
    my $obj = shift || return error ('No Object!' );

    # globals
    my $channels;

    my $prompt = "XXV> ";

    # make socket
	my $socket = IO::Socket::INET->new(
		Listen		=> $obj->{Clients} || 5,
		LocalPort	=> $obj->{Port}    || 8001,
		Reuse		=> 1);
	warn sprintf("[Fatal] Cannot build initial socket for %s on port %d. [%s]\n",
	        $obj->{Name}, $obj->{Port}, $!) unless $socket;

    # install an initial watcher
    Event->io(
        fd => $socket,
        cb => sub {
                my $watcher = shift;

                # make "channel" number
                my $channel=++$channels;

                # accept client
                my $client=$socket->accept;
                warn "[Error] Cannot connect telnet to new client.\n" and return unless $client;
                $client->autoflush;

                my $console = XXV::OUTPUT::Console->new(
                    -handle => $client,
                    -dbh    => $obj->{dbh},
                    -paths=> $obj->{paths},
                );

                # install a communicator
                Event->io(
                    fd => $client,
                    cb => sub {
                            my $watcher = shift;

                            # read new line and report it
                            my $handle=$watcher->w->fd;

                            my $line=<$handle>;
                            unless($line) {
                                delete $obj->{USER};
                                $watcher->w->cancel;
                                $handle->close();
                                undef $watcher;
                                return 1;
                            }
                            $line =~ s/[\r|\n]//sig
                                if(defined $line);

                            $obj->handleInput($watcher, $console, $line);
                            # Prompt
                            $client->print($prompt) if($client->opened);
                        },
                );

                # welcome
                $client->print(sprintf(gettext("Welcome to XXV System Ver.: %s.\nThis is session %s.\n"),$obj->{MOD}->{Version},$channel));

                my $userMod = main::getModule('USER');
                unless(exists $obj->{USER} or $userMod->{Active} ne 'y') {
                    # Login
                    my $data = $console->login(gettext("Welcome to xxv System. Please Login:"));
                    my $user = $userMod->check($data->{Name}, $data->{Password});
                    if(exists $user->{Name}) {
                        $obj->{USER} = $user;
                    } else {
                        $console->message(gettext("Sorry, but permission denied!"));
                        delete $obj->{USER};
                        $client->close;
                        return 1;
                    }
                }

                $client->print($prompt);

            },
    );
}

# ------------------
sub usage {
# ------------------
    my $obj         = shift || return error ('No Object!' );
    my $watcher     = shift || return error ('No Watcher!');
    my $console     = shift || return error ('No Console');
    my $modulename  = shift || 0;
    my $hint        = shift || '';
    my $user        = shift || 0;

    my $u = main::getModule('USER');
    unless($user) {
        my $loginObj = $obj;
        $loginObj = main::getModule('HTTPD')
                if ($console->{TYP} eq 'HTML') ;
        $loginObj = main::getModule('WAPD')
                if ($console->{TYP} eq 'WML') ;
        $user = $loginObj->{USER};
    }

    my $ret;
    push(@$ret, sprintf(gettext("%sThis is the XXV-%s-Server.\nPlease use the following commands:\n"),
        ($hint ? "$hint\n\n" : ''), $console->typ));

    my $mods = main::getModules();
    # Search for command and display the Description
    foreach my $modName (sort keys %{$mods}) {
        my $modCfg = $mods->{$modName}->{MOD};
        next if($modulename and uc($modulename) ne $modCfg->{Name});
        foreach my $cmdName (sort keys %{$modCfg->{Commands}}) {
            push(@$ret,
                [
                    (split('::', $modName))[-1],
                    $modCfg->{Commands}->{$cmdName}->{short},
                    $cmdName,
                    $modCfg->{Commands}->{$cmdName}->{description},
                ]
            ) if(($u->{Active} ne 'y') || $u->allowCommand($modCfg, $cmdName, $user, "1"));
        }
    }
    $console->menu(
        $ret,
        {
            TIMERS   => $mods->{'XXV::MODULES::EPG'}->{timers},
            CHANNELS => $mods->{'XXV::MODULES::CHANNELS'}->ChannelArray('Name'),
            CONFIGS  => [ sort keys %$mods ],
        },
    );
}

# ------------------
sub handleInput {
# ------------------
    my $obj     = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $line    = shift || return;
    my $user    = shift || $obj->{USER};

    $obj->{USER} ||= $user;

    my ($ucmd, $udata) = ($1, $2) if($line =~ /(\S+)\s*(.*)/sig);

    my $mods = main::getModules();

    # Checks Commands
    $obj->checkCommands()
        unless(defined $obj->{Check});

    my $ok = 0;
    foreach my $modName (keys %{$mods}) {
        my $modCfg = $mods->{$modName}->{MOD};
        foreach my $cmdName (sort keys %{$modCfg->{Commands}}) {
            if(lc($ucmd) eq $cmdName or lc($ucmd) eq $modCfg->{Commands}->{$cmdName}->{short}) {
                $ok++;
                $console->{call} = $cmdName;

                # Check Userlevel and Permissions
                my $u = main::getModule('USER');
                if($u->allowCommand($modCfg, $cmdName, $user)) {
                    $modCfg->{Commands}->{$cmdName}->{callback}($watcher, $console, $udata);
                } else {
                    $console->message(gettext('You have no permissions on this command!'));
                }
            }
        }
    }

    unless($ok) {
        $obj->usage($watcher, $console, undef,  sprintf(gettext("I don't understand the command '%s' \n"),$ucmd), $user);
    }
}

# ------------------
sub checkCommands {
# ------------------
    my $obj     = shift || return error ('No Object!' );
    my $mods    = main::getModules();

    my $shorts = {};
    foreach my $modName (keys %{$mods}) {
        my $modCfg = $mods->{$modName}->{MOD};
        foreach my $cmdName (sort keys %{$modCfg->{Commands}}) {
            my $short = $modCfg->{Commands}->{$cmdName}->{short};
            if(defined $shorts->{$short} ) {
                warn sprintf("[ERROR] In %s::%s double short name %s, also in %s!",
                    $modName, $cmdName, $short, $shorts->{$short});
            } else {
                $shorts->{$short} = $modName.'::'.$cmdName;
            }
        }
    }
    $obj->{Check} = 1;
}

1;

==> vdr-xxv_0.52.orig.tar.gz: xxv-0.52/lib/XXV/MODULES/TELNET.pm <==
package XXV::MODULES::TELNET;

use strict;

use XXV::OUTPUT::Console;
use Tools;
use Locale::gettext;

# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'TELNET',
        Prereq => {
            'IO::Socket::INET'  => 'Object interface for AF_INET domain sockets ',
            'Module::Reload'    => 'Reload %INC files when updated on disk ',
        },
        Description => gettext('This module is a multisession telnet server.'),
        Version => '0.01',
        Date => '18.08.2004',
        Author => 'xpix',
        Preferences => {
            active => {
                description => gettext('Activate this service'),
                default     => 'y',
                type        => 'confirm',
                required    => gettext('This is required!'),
            },
            Clients => {
                description => gettext('Maximum number of allowed connections at a time'),
                default     => 5,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
            Port => {
                description => gettext('Portnumber to connect the client'),
                default     => 8005,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
        },
        Commands => {
            help => {
                description => gettext("This will display all commands or the helptext from the 'module name'"),
                short       => 'h',
                callback    => sub{
                    return $obj->usage(@_);
                },
            },
            quit => {
                description => gettext("This will exit the telnet session"),
                short       => 'q',
                callback    => sub{
                    my ($w, $c, $l) = @_;
                    lg "Closed telnet session.\n";
                    $c->message(gettext("Closed session."));
                    $w->w->fd->close;
                    $w->w->cancel;
                },
            },
            bye => {
                description => gettext("This will exit the xxv system."),
                short       => 'x',
                callback    => sub{
                    my ($w, $c, $l) = @_;
        		    my $answer;
        		    my $questions = [
            		    'really' =>	{
                                    typ => 'confirm',
                                    msg => gettext("Are you sure to exit the XXV System?"),
                                    def => 'n'}
        		    ];
                    $answer = $c->question(gettext("This will exit the xxv system."),$questions,$answer);
                    if($answer->{really} eq 'y') {
                        $w->w->fd->close;
                        $w->w->cancel;
                        lg "Closed session and exit.\n";
                        main::quit;
                    }
                },
        		Level   => 'admin'
            },
            reload => {
                description => gettext("This will reload all Modules."),
                short       => 'rel',
                callback    => sub{
                    my ($w, $c, $l) = @_;
                    $Module::Reload::Debug = 2;
                    Module::Reload->check;
                    $c->message(gettext("Modules reloaded."));
                },
		        Level   => 'admin'
            },
            pureperl => {
                hidden      => 'yes',
                callback    => sub{ $obj->pureperl(@_) },
            },
        },
    };
    return $args;
}

# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_} || $self->{MOD}->{Preferences}->{$_}->{default}
    } keys %{$self->{MOD}->{Preferences}};

    # Try to use the Requirments
    map {
        eval "use $_";
        return panic("\nCan not load Module: $_\nPlease install this module on your System:\nperl -MCPAN -e 'install $_'") if($@);
    } keys %{$self->{MOD}->{Prereq}};

    # read the DB Handle
    $self->{dbh} = delete $attr{'-dbh'};

    # The Initprocess
    $self->init or return error('Problem to initialze Module');

	return $self;
}


# ------------------
sub init {
# ------------------
    my $obj = shift || return error ('No Object!' );

    # globals
    my $channels;

    my $prompt = "XXV> ";

    # make socket
	my $socket = IO::Socket::INET->new(
		Listen		=> $obj->{Clients} || 5,
		LocalPort	=> $obj->{Port}    || 8001,
		Reuse		=> 1
    ) or return error("Can't create Socket: $!");

    # install an initial watcher
    Event->io(
        fd => $socket,
        prio => -1,  # -1 very hard ... 6 very low
        cb => sub {
                my $watch = shift;

                # make "channel" number
                my $channel=++$channels;

                # accept client
                my $client=$socket->accept;
                panic "Can't connect telnet to new client.\n" and return unless $client;
                $client->autoflush;

                my $console = XXV::OUTPUT::Console->new(
                    -handle => $client,
                    -dbh    => $obj->{dbh},
                    -paths  => $obj->{paths},
                );

                # install a communicator
                Event->io(
                    fd => $client,
                    prio => -1,  # -1 very hard ... 6 very low
                    cb => sub {
                            my $watcher = shift;

                            # read new line and report it
                            my $handle=$watcher->w->fd;

                            my $line=<$handle>;
                            unless($line) {
                                $watcher->w->cancel;
                                $handle->close();
                                undef $watcher;
                                return 1;
                            }
                            $line =~ s/[\r|\n]//sig
                                if(defined $line);

                            $obj->handleInput($watcher, $console, $line);
                            # Prompt
                            $client->print($prompt) if($client->opened);
                        },
                );

                # welcome
                $client->print(sprintf(gettext("Welcome to XXV System Ver.: %s.\r\nThis is session %s.\r\n"),$obj->{MOD}->{Version},$channel));

                my $userMod = main::getModule('USER');
                unless(exists $console->{USER} or $userMod->{active} ne 'y') {
                    # Login
                    my $data;
                    if($userMod->_checkIp($client)) {
                        $console->message(gettext("Welcome to xxv System."));
                        $data->{Name} = 'no';
                        $data->{Password} = 'no';
                    } else {
                        $data = $console->login(gettext("Welcome to xxv System. Please Login:"));
                    }
                    my $user = $userMod->check($client, $data->{Name}, $data->{Password});
                    if(exists $user->{Name}) {
                        $console->{USER} = $user;
                    } else {
                        $console->message(gettext("Sorry, but permission denied!"));
                        $client->close;
                        return 1;
                    }
                }

                $client->print($prompt);

            },
    ) if($obj->{active} eq 'y');

    return 1;

}

# ------------------
sub usage {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $modulename = shift || 0;
    my $hint = shift || '';
    my $user = shift || $console->{USER};

    my $u = main::getModule('USER');
    unless($user) {
        my $loginObj = $obj;
        $loginObj = main::getModule('HTTPD')
                if ($console->{TYP} eq 'HTML') ;
        $loginObj = main::getModule('WAPD')
                if ($console->{TYP} eq 'WML') ;
        $user = $loginObj->{USER};
    }

    my $ret;
    push(@$ret, sprintf(gettext("%sThis is the XXV-%s-Server.\nPlease use the following commands:\n"),
        ($hint ? "$hint\n\n" : ''), $console->typ));

    my $mods = main::getModules();
    my @realModName;

    # Search for command and display the Description
    foreach my $modName (sort keys %{$mods}) {
        my $modCfg = $mods->{$modName}->{MOD};
        push(@realModName, $mods->{$modName}->{MOD}->{Name});
        next if($modulename and uc($modulename) ne $modCfg->{Name});
        foreach my $cmdName (sort keys %{$modCfg->{Commands}}) {
            push(@$ret,
                [
                    (split('::', $modName))[-1],
                    $modCfg->{Commands}->{$cmdName}->{short},
                    $cmdName,
                    $modCfg->{Commands}->{$cmdName}->{description},
                ]
            ) if(! $modCfg->{Commands}->{$cmdName}->{hidden} and ($u->{active} ne 'y') || $u->allowCommand($modCfg, $cmdName, $user, "1"));
        }
    }

    $console->menu(
        $ret,
        {
            periods  => $mods->{'XXV::MODULES::EPG'}->{periods},
            CHANNELS => $mods->{'XXV::MODULES::CHANNELS'}->ChannelArray('Name'),
            CONFIGS  => [ sort @realModName ],
        },
    );
}

# ------------------
sub handleInput {
# ------------------
    my $obj     = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $line    = shift || return;
    my $user    = shift || $console->{USER};

    my ($ucmd, $udata) = ($1, $2) if($line =~ /(\S+)\s*(.*)/sig);

    # Test the command on exists, permissions and so on
    my $u = main::getModule('USER');
    my ($cmdobj, $cmdname, $shorterr, $err) = $u->checkCommand($console, $ucmd);
    $console->{call} = $cmdname;
    if($cmdobj and not $shorterr) {
        $cmdobj->{callback}($watcher, $console, $udata);
    } elsif($shorterr eq 'noperm' or $shorterr eq 'noactive') {
        return $console->err($err);
    } else {
        return $obj->usage($watcher, $console, undef, $err);
    }
}

# ------------------
sub pureperl {
# ------------------
    my $obj = shift  || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $data = shift || return error ('No Data!' );

    my $erg = eval $data;
    return $console->msg($erg, $@)
        if(ref $console);
}


1;
